searchData={"items":[{"type":"module","title":"Aino","doc":"Aino, an experimental HTTP framework\n\nTo load Aino, add to your supervision tree.\n\n`callback`, `otp_app`, `host`, and `port` are required options.\n\n`environment` and `config` are optional and passed into your token\nwhen it's created on each request.\n\n```elixir\n  aino_config = %Aino.Config{\n    callback: Example.Web.Handler,\n    otp_app: :example,\n    scheme: config.scheme,\n    host: config.host,\n    port: config.port,\n    url_port: config.url_port,\n    url_scheme: config.url_scheme,\n    environment: config.environment,\n    config: %{}\n  }\n\n  children = [\n    {Aino.Supervisor, aino_config}\n  ]\n```\n\nThe `callback` should be an `Aino.Handler`, which has a single `handle/1` function that\nprocesses the request.\n\n`otp_app` should be the atom of your OTP application, e.g. `:example`.\n\n`host` and `port` are used for binding and booting the webserver, and\nas default assigns for the token when rendering views.\n\n`environment` is the environment the application is running under, similar to `Mix.env()`.\n\n`config` is a simple map that is passed into the token when created, example\nvalues to pass through this config map is your session salt.","ref":"Aino.html"},{"type":"module","title":"Aino.Assets","doc":"Generate a manifest file and handles path translation","ref":"Aino.Assets.html"},{"type":"function","title":"Aino.Assets.asset_path/1","doc":"Load an asset path from the manifest\n\nIf the path is not present in the manifest, the same path is returned","ref":"Aino.Assets.html#asset_path/1"},{"type":"function","title":"Aino.Assets.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Aino.Assets.html#child_spec/1"},{"type":"function","title":"Aino.Assets.file_from_asset/1","doc":"Search the loaded manifest by asset path\n\nFind the matching actual file path. If none is found, the asset path is returned.","ref":"Aino.Assets.html#file_from_asset/1"},{"type":"function","title":"Aino.Assets.generate_manifest/1","doc":"Generate a manifest file for the application","ref":"Aino.Assets.html#generate_manifest/1"},{"type":"function","title":"Aino.Assets.path/1","doc":"","ref":"Aino.Assets.html#path/1"},{"type":"behaviour","title":"Aino.ChunkedHandler","doc":"Chunked Response Handler\n\nIn order to [send chunked data](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Transfer-Encoding)\nwith Aino, you must use a ChunkedHandler.\n\nTo use chunked response, you must return `chunk: true` and `handler: YourHandler` in the token.\n\n# EventStream Example\n\nIn the example below, a Ping route starts a chunked response.\n\n```elixir\ndefmodule Web.Ping do\n  def index(token) do\n    token\n    |> Token.response_status(200)\n    |> Token.response_header(\"Content-Type\", \"text/event-stream\")\n    |> Map.put(:chunk, true)\n    |> Map.put(:handler, Web.Ping.Handler)\n  end\nend\n\ndefmodule Web.Ping.Handler do\n  @behaviour Aino.ChunkedHandler\n\n  @impl true\n  def init(token) do\n    :timer.send_interval(1000, :ping)\n\n    {:ok, token}\n  end\n\n  @impl true\n  def handle(:ping, token) do\n    json = Jason.encode!(%{time: DateTime.utc_now()})\n    response = \"event: ping\ndata: #{json}\n\n\"\n    {:ok, response, token}\n  end\nend\n```","ref":"Aino.ChunkedHandler.html"},{"type":"callback","title":"Aino.ChunkedHandler.handle/2","doc":"Handle incoming messages\n\nCalled when any incoming messages are sent to the GenServer processing your chunked response","ref":"Aino.ChunkedHandler.html#c:handle/2"},{"type":"callback","title":"Aino.ChunkedHandler.init/1","doc":"Initialize your handler\n\nCalled when the chunked response initializes.","ref":"Aino.ChunkedHandler.html#c:init/1"},{"type":"module","title":"Aino.Config","doc":"Config for `Aino` when launching in a supervision tree\n\n```elixir\n  aino_config = %Aino.Config{\n    callback: Example.Web.Handler,\n    otp_app: :example,\n    scheme: config.scheme,\n    host: config.host,\n    port: config.port,\n    url_port: config.url_port,\n    url_scheme: config.url_scheme,\n    environment: config.environment,\n    config: %{}\n  }\n\n  children = [\n    {Aino.Supervisor, aino_config}\n  ]\n```","ref":"Aino.Config.html"},{"type":"module","title":"Aino.Event","doc":"An outgoing event, primarily for [Server Sent Events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events)","ref":"Aino.Event.html"},{"type":"behaviour","title":"Aino.Handler","doc":"Process an incoming request from Aino\n\nThe `handle/1` function is passed an `Aino.Token`.\n\nThe handler _must_ return a token that contains three keys to return a response:\n\n- `:response_status`\n- `:response_headers`\n- `:response_body`\n\nIf the token does not contain these three keys, a 500 error is returned.\n\nInside your handler, you may wish to use several `Aino.Middleware` including\n`Aino.Middleware.common/0`.","ref":"Aino.Handler.html"},{"type":"callback","title":"Aino.Handler.handle/1","doc":"Process an incoming request from Aino\n\nThe argument is an `Aino.Token`.","ref":"Aino.Handler.html#c:handle/1"},{"type":"module","title":"Aino.Request","doc":"A generic request for usage in Aino\nConverted from the adapter's request","ref":"Aino.Request.html"},{"type":"type","title":"Aino.Request.t/0","doc":"","ref":"Aino.Request.html#t:t/0"},{"type":"module","title":"Aino.Session.CSRF","doc":"Session Middleware for handling CSRF validation\n\nExample for using CSRF:\n\n```elixir\nmiddleware = [\n  Aino.Middleware.common(),\n  &Aino.Session.config(&1, %Aino.Session.Cookie{key: \"key\", salt: \"salt\"}),\n  &Aino.Session.decode/1,\n  &Aino.Session.Flash.load/1,\n  &Aino.Middleware.Routes.routes(&1, routes()),\n  &Aino.Middleware.Routes.match_route/1,\n  &Aino.Middleware.params/1,\n  &Aino.Session.CSRF.validate/1,\n  &Aino.Session.CSRF.generate/1,\n  &Aino.Middleware.Routes.handle_route/1,\n  &Aino.Session.encode/1,\n  &Aino.Middleware.logging/1\n]\n\nAino.Token.reduce(token, middleware)\n```\n\n`validate/1` and `generate/1` should be after `Session.load/1` and\n`Aino.Middleware.params/1` to make sure the token can be properly loaded.\n\nYour forms must now include a new hidden field named `_csrf_token`. This will\nuse the session's `_csrf_token` value.\n\n```\n \" />\n```","ref":"Aino.Session.CSRF.html"},{"type":"function","title":"Aino.Session.CSRF.generate/1","doc":"Generate a token and store in the session","ref":"Aino.Session.CSRF.html#generate/1"},{"type":"function","title":"Aino.Session.CSRF.validate/1","doc":"Validate the token is present if a POST or PUT","ref":"Aino.Session.CSRF.html#validate/1"},{"type":"module","title":"Aino.Supervisor","doc":"Small supervisor to handle launching Aino","ref":"Aino.Supervisor.html"},{"type":"function","title":"Aino.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Aino.Supervisor.html#child_spec/1"},{"type":"function","title":"Aino.Supervisor.init/1","doc":"","ref":"Aino.Supervisor.html#init/1"},{"type":"function","title":"Aino.Supervisor.start_link/1","doc":"","ref":"Aino.Supervisor.html#start_link/1"},{"type":"task","title":"mix aino.assets","doc":"Generate a manifest file for assets","ref":"Mix.Tasks.Aino.Assets.html"},{"type":"task","title":"mix aino.server","doc":"Run the Aino Server\n\nAn alias for `mix run --no-halt` that is IEx aware","ref":"Mix.Tasks.Aino.Server.html"},{"type":"module","title":"Aino.Middleware","doc":"Middleware functions for processing a request into a response\n\nIncluded in Aino are common functions that deal with requests, such\nas parsing the POST body for form data or parsing query/path params.","ref":"Aino.Middleware.html"},{"type":"function","title":"Aino.Middleware.adjust_method/1","doc":"Adjust the request's method based on a special post body parameter\n\nSince browsers cannot perform DELETE/PUT/PATCH requests, allow overriding the method\nbased on the `_method` parameter.\n\nPOST body data *must* be parsed before being able to adjust the method.\n\n    iex> token = %{method: :post, parsed_body: %{\"_method\" => \"delete\"}}\n    iex> token = Middleware.adjust_method(token)\n    iex> token.method\n    :delete\n\n    iex> token = %{method: :post, parsed_body: %{\"_method\" => \"patch\"}}\n    iex> token = Middleware.adjust_method(token)\n    iex> token.method\n    :patch\n\n    iex> token = %{method: :post, parsed_body: %{\"_method\" => \"put\"}}\n    iex> token = Middleware.adjust_method(token)\n    iex> token.method\n    :put\n\nIgnored adjustments\n\n    iex> token = %{method: :post, parsed_body: %{\"_method\" => \"new\"}}\n    iex> token = Middleware.adjust_method(token)\n    iex> token.method\n    :post\n\n    iex> token = %{method: :get}\n    iex> token = Middleware.adjust_method(token)\n    iex> token.method\n    :get","ref":"Aino.Middleware.html#adjust_method/1"},{"type":"function","title":"Aino.Middleware.assets/1","doc":"Serve static assets\n\nLoads static files out the `priv/static` folder for your OTP app. Looks for\nthe path to begin with `/assets` and everything afterwards is used as a file.\nIf the file exists, it is returned with a `200` status code.\n\nThe content type will be guessed at using the `MIME` hex package.\n\nExample: `/assets/js/app.js` will look for a file in `priv/static/js/app.js`","ref":"Aino.Middleware.html#assets/1"},{"type":"function","title":"Aino.Middleware.common/0","doc":"Common middleware that process low level request data\n\nProcesses the request:\n- method\n- path\n- headers\n- query parameters\n- parses response body\n- parses cookies","ref":"Aino.Middleware.html#common/0"},{"type":"function","title":"Aino.Middleware.cookies/1","doc":"Processes the `Cookie` request header\n\nDefaults to an empty map if no `Cookie` header is present.\n\nStores cookies as a map in the key `:cookies`","ref":"Aino.Middleware.html#cookies/1"},{"type":"function","title":"Aino.Middleware.headers/1","doc":"Processes request headers\n\nDowncases all of the headers and stores in the key `:headers`","ref":"Aino.Middleware.html#headers/1"},{"type":"function","title":"Aino.Middleware.logging/1","doc":"","ref":"Aino.Middleware.html#logging/1"},{"type":"function","title":"Aino.Middleware.method/1","doc":"Stores the request method on the token\n\nDowncases and converts to an atom on the key `:method`\n\n    iex> request = %Aino.Request{method: :GET}\n    iex> token = %{request: request}\n    iex> token = Middleware.method(token)\n    iex> token.method\n    :get","ref":"Aino.Middleware.html#method/1"},{"type":"function","title":"Aino.Middleware.params/1","doc":"Merge params into a single map\n\nMerges in the following order:\n- Path params\n- Query params\n- POST body","ref":"Aino.Middleware.html#params/1"},{"type":"function","title":"Aino.Middleware.path/1","doc":"Stores the request path on the token on the key `:path`\n\n    iex> request = %Aino.Request{path: \"/orders/10\"}\n    iex> token = %{request: request}\n    iex> token = Middleware.path(token)\n    iex> token.path\n    \"/orders/10\"","ref":"Aino.Middleware.html#path/1"},{"type":"function","title":"Aino.Middleware.query_params/1","doc":"Stores query parameters on the token\n\nConverts map and stores on the key `:query_params`\n\n    iex> request = %Aino.Request{query_params: %{\"key[]\" => \"value\"}}\n    iex> token = %{request: request}\n    iex> token = Middleware.query_params(token)\n    iex> token.query_params\n    %{\"key\" => [\"value\"]}","ref":"Aino.Middleware.html#query_params/1"},{"type":"function","title":"Aino.Middleware.request_body/1","doc":"Processes the request body\n\nOnly if the request should have a body (e.g. POST requests)\n\nHandles the following content types:\n- `application/x-www-form-urlencoded`\n- `application/json`","ref":"Aino.Middleware.html#request_body/1"},{"type":"module","title":"Aino.Middleware.Development","doc":"Development only middleware\n\nThese should *not* be used in production.","ref":"Aino.Middleware.Development.html"},{"type":"function","title":"Aino.Middleware.Development.inspect/2","doc":"Debug log a key on the token","ref":"Aino.Middleware.Development.html#inspect/2"},{"type":"function","title":"Aino.Middleware.Development.recompile/1","doc":"Recompiles the application","ref":"Aino.Middleware.Development.html#recompile/1"},{"type":"behaviour","title":"Aino.Middleware.Parsers","doc":"Behaviour for parsing request bodys","ref":"Aino.Middleware.Parsers.html"},{"type":"callback","title":"Aino.Middleware.Parsers.parse/1","doc":"Parse the request body for a token","ref":"Aino.Middleware.Parsers.html#c:parse/1"},{"type":"module","title":"Aino.Middleware.Parsers.FormURLEncoded","doc":"Parse `application/x-www-form-urlencoded` request bodies","ref":"Aino.Middleware.Parsers.FormURLEncoded.html"},{"type":"module","title":"Aino.Middleware.Parsers.ParamKeys","doc":"Common code for parsing param maps","ref":"Aino.Middleware.Parsers.ParamKeys.html"},{"type":"function","title":"Aino.Middleware.Parsers.ParamKeys.parse/1","doc":"Converts a flat map with special keys into a deep map","ref":"Aino.Middleware.Parsers.ParamKeys.html#parse/1"},{"type":"module","title":"Aino.Middleware.Routes","doc":"An Aino set of middleware for dealing with routes and routing","ref":"Aino.Middleware.Routes.html"},{"type":"module","title":"Examples - Aino.Middleware.Routes","doc":"To use the routes middleware together, see the example below.\n\n```elixir\ndef routes() do\n  [\n    get(\"/orders\", &Orders.index/1, as: :orders),\n    get(\"/orders/:id\", [&Orders.authorize/1, &Order.show/1], as: :order),\n    post(\"/orders\", &Orders.create/1),\n    post(\"/orders/:id\", [&Orders.authorize/1, &Order.update/1])\n  ]\nend\n\ndef handle(token) do\n  middleware = [\n    Aino.Middleware.common(),\n    &Aino.Middleware.Routes.routes(&1, routes()),\n    &Aino.Middleware.Routes.match_route/1,\n    &Aino.Middleware.params/1,\n    &Aino.Middleware.Routes.handle_route/1\n  ]\n\n  Aino.Token.reduce(token, middleware)\nend\n```\n\nIn the example above you can see why `match_route/1` and `handle_route/1` are\nseparate functions, you can perform other middleware in between the two. In this\nexample, params are merged together via `Aino.Middleware.params/1` before\nhandling the route.","ref":"Aino.Middleware.Routes.html#module-examples"},{"type":"macro","title":"Aino.Middleware.Routes.compile/1","doc":"Configure routes for the handler\n\nFor use with a `__MODULE__.Routes` module to hold for route helper functions\n\nWhen defining routes, provide the `:as` option to have `_path` and `_url` functions\ngenerated for you. E.g. `as: :sign_in` will generate `Routes.sign_in_path/2` and\n`Routes.sign_in_url/2`.\n\nNote that when defining routes, you must only define one `:as` a particular atom. For\ninstance, if you have multiple routes pointing at the url `/orders/:id`, you should only\nadd `as: :order` to the first route.\n\n```elixir\ndef routes() do\n  [\n    get(\"/\", &MyApp.Web.Page.root/1, as: :root),\n    get(\"/sign-in\", &MyApp.Web.Session.show/1, as: :sign_in),\n    post(\"/sign-in\", &MyApp.Web.Session.create/1),\n    delete(\"/sign-out\", &MyApp.Web.Session.delete/1, as: :sign_out),\n    get(\"/orders\", &MyApp.Web.Orders.index/1, as: :orders),\n    get(\"/orders/:id\", &MyApp.Web.Orders.show/1, as: :order)\n  ]\nend\n\ndefmodule MyApp.Web.Handler.Routes do\n  require Aino.Middleware.Routes\n\n  Aino.Middleware.Routes.compile(routes())\nend\n```","ref":"Aino.Middleware.Routes.html#compile/1"},{"type":"function","title":"Aino.Middleware.Routes.delete/3","doc":"Create a DELETE route","ref":"Aino.Middleware.Routes.html#delete/3"},{"type":"function","title":"Examples - Aino.Middleware.Routes.delete/3","doc":"```elixir\nroutes = [\n  delete(\"/orders/:id\", [&Orders.authorize/1, &Order.delete/1], as: :order)\n]\n```","ref":"Aino.Middleware.Routes.html#delete/3-examples"},{"type":"function","title":"Aino.Middleware.Routes.get/3","doc":"Create a GET route","ref":"Aino.Middleware.Routes.html#get/3"},{"type":"function","title":"Examples - Aino.Middleware.Routes.get/3","doc":"```elixir\nroutes = [\n  get(\"/orders\", &Orders.index/1, as: :orders),\n  get(\"/orders/:id\", [&Orders.authorize/1, &Order.show/1], as: :order)\n]\n```","ref":"Aino.Middleware.Routes.html#get/3-examples"},{"type":"function","title":"Aino.Middleware.Routes.handle_route/1","doc":"Run the matched route from `match_route/1`\n\nIf no route is present, nothing happens. If a route is present, the\nmiddleware stored on the token from the matched request is reduced over.","ref":"Aino.Middleware.Routes.html#handle_route/1"},{"type":"function","title":"Aino.Middleware.Routes.match_route/1","doc":"Matches the request against routes on the token\n\n_Must_ have routes set via `routes/2` before running this middleware.\n\nYou _should_ run `handle_route/1` after matching the route, otherwise\nthe route is not run.\n\nAdds the following keys to the token `[:path_params, :route_middleware]`","ref":"Aino.Middleware.Routes.html#match_route/1"},{"type":"function","title":"Aino.Middleware.Routes.post/3","doc":"Create a POST route","ref":"Aino.Middleware.Routes.html#post/3"},{"type":"function","title":"Examples - Aino.Middleware.Routes.post/3","doc":"```elixir\nroutes = [\n  post(\"/orders\", &Orders.create/1, as: :orders),\n  post(\"/orders/:id\", [&Orders.authorize/1, &Order.update/1], as: :order)\n]\n```","ref":"Aino.Middleware.Routes.html#post/3-examples"},{"type":"function","title":"Aino.Middleware.Routes.put/3","doc":"Create a PUT route","ref":"Aino.Middleware.Routes.html#put/3"},{"type":"function","title":"Examples - Aino.Middleware.Routes.put/3","doc":"```elixir\nroutes = [\n  put(\"/orders\", &Orders.create/1, as: :orders),\n  post(\"/orders/:id\", [&Orders.authorize/1, &Order.update/1], as: :order)\n]\n```","ref":"Aino.Middleware.Routes.html#put/3-examples"},{"type":"function","title":"Aino.Middleware.Routes.routes/2","doc":"Set routes for the token\n\nAdds the following keys to the token `[:routes]`","ref":"Aino.Middleware.Routes.html#routes/2"},{"type":"module","title":"Aino.Token","doc":"The token is what flows through the entire web request\n\nThis module contains helper functions for dealing with the token, setting\ncommon fields for responses or looking up request fields.\n\nAt the end of a middleware chain, the token _must_ contain three keys:\n\n- `:response_status`\n- `:response_headers`\n- `:response_body`\n\nThese keys are used for generating the request's response.","ref":"Aino.Token.html"},{"type":"function","title":"Aino.Token.assign/3","doc":"Add a key/value pair to the `assigns` map on a token\n\n    iex> token = %{assigns: %{}}\n    iex> Token.assign(token, :name, \"User\")\n    %{assigns: %{name: \"User\"}}","ref":"Aino.Token.html#assign/3"},{"type":"function","title":"Aino.Token.from_request/1","doc":"Start a token from an `:elli` request\n\nThe token gains the keys `[:request]`\n\n    iex> request = %Aino.Request{}\n    iex> token = Token.from_request(request)\n    iex> token.request == request\n    true","ref":"Aino.Token.html#from_request/1"},{"type":"function","title":"Aino.Token.reduce/2","doc":"Reduce a token over a set of middleware.\n\nTakes a list of middleware, that may be either another list of middleware or\na function that has an arity of 1.\n\nOptionally you can use a two element tuple, the first element is a middleware\nfunction reference, with the second element being a list of options.\n\nOptions available for middleware during runtime are:\n- `ignore_halt`, default false, if set to true the middleware will always run\n  and not be short circuited by a halted token.\n\nFor example\n\n```elixir\nmiddleware = [\n  Aino.Middleware.common(),\n  &Aino.Middleware.Routes.routes(&1, routes),\n  &Aino.Middleware.Routes.match_route/1,\n  &Aino.Middleware.params/1,\n  &Aino.Middleware.Routes.handle_route/1,\n  {&Aino.Middleware.logging/1, ignore_halt: true}\n]\n\nreduce(token, middleware)\n```","ref":"Aino.Token.html#reduce/2"},{"type":"function","title":"Aino.Token.request_header/2","doc":"Get a request header from the token\n\nThis must be used with `Aino.Middleware.headers/1` since that middleware sets\nup the token to include a `:headers` key that is downcased.\n\nThe request header that is searched for is lower cased and compared against\nrequest headers, filtering down to matching headers.\n\n    iex> token = %{headers: [{\"content-type\", \"text/html\"}, {\"location\", \"/\"}]}\n    iex> Token.request_header(token, \"Content-Type\")\n    [\"text/html\"]","ref":"Aino.Token.html#request_header/2"},{"type":"function","title":"Aino.Token.response_body/2","doc":"Set the response body\n\nWhen setting a response body, you _should_ also set a `Content-Type` header.\nThis way the client can know what type of data it received.\n\nThe token gains or modifies the keys `[:response_body]`\n\n    iex> token = %{}\n    iex> Token.response_body(token, \"html\")\n    %{response_body: \"html\"}","ref":"Aino.Token.html#response_body/2"},{"type":"function","title":"Aino.Token.response_header/2","doc":"Get a response header from the token\n\nThis must be used with `Aino.Middleware.headers/1` since that middleware sets\nup the token to include a `:headers` key that is downcased.\n\nThe response header that is searched for is lower cased and compared against\nresponse headers, filtering down to matching headers.\n\n    iex> token = %{response_headers: [{\"content-type\", \"text/html\"}, {\"location\", \"/\"}]}\n    iex> Token.response_header(token, \"Content-Type\")\n    [\"text/html\"]","ref":"Aino.Token.html#response_header/2"},{"type":"function","title":"Aino.Token.response_header/3","doc":"Append a response header to the token\n\nResponse headers default to an empty list if this is the first header set\n\nThe token gains or modifies the keys `[:response_headers]`\n\n    iex> token = %{}\n    iex> Token.response_header(token, \"Content-Type\", \"application/json\")\n    %{response_headers: [{\"Content-Type\", \"application/json\"}]}\n\n    iex> token = %{response_headers: [{\"Content-Type\", \"text/html\"}]}\n    iex> Token.response_header(token, \"Location\", \"/\")\n    %{response_headers: [{\"Content-Type\", \"text/html\"}, {\"Location\", \"/\"}]}","ref":"Aino.Token.html#response_header/3"},{"type":"function","title":"Aino.Token.response_headers/2","doc":"Set all response headers on the token\n\nIf response headers are present, they are cleared. This directly sets the\n`:response_headers` key on the token.\n\nThe token gains or modifies the keys `[:response_headers]`\n\n    iex> token = %{}\n    iex> Token.response_headers(token, [{\"Content-Type\", \"application/json\"}])\n    %{response_headers: [{\"Content-Type\", \"application/json\"}]}\n\n    iex> token = %{response_headers: [{\"Content-Type\", \"text/html\"}]}\n    iex> Token.response_headers(token, [{\"Location\", \"/\"}])\n    %{response_headers: [{\"Location\", \"/\"}]}","ref":"Aino.Token.html#response_headers/2"},{"type":"function","title":"Aino.Token.response_status/2","doc":"Set a response status on the token\n\nThe token gains the keys `[:response_status]`\n\n    iex> token = %{}\n    iex> Token.response_status(token, 200)\n    %{response_status: 200}","ref":"Aino.Token.html#response_status/2"},{"type":"type","title":"Aino.Token.t/0","doc":"","ref":"Aino.Token.html#t:t/0"},{"type":"module","title":"Aino.Token.Response","doc":"Shortcuts for returning common responses\n\nHTML, redirecting, etc","ref":"Aino.Token.Response.html"},{"type":"function","title":"Aino.Token.Response.html/2","doc":"Sets token fields to render the response body as html\n\n    iex> token = %{}\n    iex> Token.Response.html(token, \"HTML Body\")\n    %{\n      response_headers: [{\"Content-Type\", \"text/html\"}],\n      response_body: \"HTML Body\"\n    }","ref":"Aino.Token.Response.html#html/2"},{"type":"function","title":"Aino.Token.Response.redirect/2","doc":"Sets the required token fields be a redirect.\n\n    iex> token = %{}\n    iex> Token.Response.redirect(token, \"/\")\n    %{\n      response_status: 302,\n      response_headers: [{\"Content-Type\", \"text/html\"}, {\"Location\", \"/\"}],\n      response_body: \"Redirecting...\"\n    }","ref":"Aino.Token.Response.html#redirect/2"},{"type":"module","title":"Aino.View","doc":"Compile templates into modules with a render function\n\n```elixir\ncompile [\n  \"lib/app/templates/folder/index.html.eex\"\n]\n```\n\nGenerates the following render functions:\n\n```elixir\ndef simple_render(\"index.html\", assigns) do\n  # compiled index.html.eex\nend\n\ndef render(token, \"index.html\", assigns) do\n  # calls simple_render(\"index.html\", assigns)\n  # assigns to the `response_body` field on `token`\nend\n```","ref":"Aino.View.html"},{"type":"macro","title":"Aino.View.compile/1","doc":"Compile a list of templates into render functions","ref":"Aino.View.html#compile/1"},{"type":"function","title":"Aino.View.compile_template/1","doc":"Compile an individual file into a quoted render function\n\nFor example, `lib/app/templates/index.html.eex` would\ngenerate the following:\n\n```elixir\ndef render(\"index.html\", assigns) do\n  # compiled index.html.eex\nend\n```","ref":"Aino.View.html#compile_template/1"},{"type":"function","title":"Aino.View.render_template/4","doc":"Manages default assigns before rendering a template with a token\n\nAssigns the response value to `response_body`","ref":"Aino.View.html#render_template/4"},{"type":"function","title":"Aino.View.safe/1","doc":"Allow a value to skip escaping for HTML","ref":"Aino.View.html#safe/1"},{"type":"module","title":"Aino.View.Engine","doc":"Aino's EEx view engine\n\nMakes text safe for rendering in a browser. Similar to `Phoenix.HTML.Engine`.","ref":"Aino.View.Engine.html"},{"type":"function","title":"Aino.View.Engine.html_escape/1","doc":"Escape special HTML characters","ref":"Aino.View.Engine.html#html_escape/1"},{"type":"protocol","title":"Aino.View.Safe","doc":"Initial protocol based off of `Phoenix.HTML.Safe`","ref":"Aino.View.Safe.html"},{"type":"function","title":"Aino.View.Safe.to_iodata/1","doc":"Convert a value to a safe value for displaying to a user\n\nHTML values that are unexpected should be escaped.","ref":"Aino.View.Safe.html#to_iodata/1"},{"type":"type","title":"Aino.View.Safe.t/0","doc":"All the types that implement this protocol.","ref":"Aino.View.Safe.html#t:t/0"},{"type":"module","title":"Aino.View.Tag","doc":"Helper functions for dealing with HTML tags","ref":"Aino.View.Tag.html"},{"type":"function","title":"Aino.View.Tag.content_tag/3","doc":"Create a new element with attributes and inner content","ref":"Aino.View.Tag.html#content_tag/3"},{"type":"module","title":"Aino.Session","doc":"Session storage","ref":"Aino.Session.html"},{"type":"function","title":"Aino.Session.config/2","doc":"Put a session configuration into the token\n\nUsed for `decode/1` and `encode/1`. The configuration should be an implementation\nof `Aino.Session.Storage`.\n\nThe following keys will be added to the token `[:session_config]`\n\n    iex> config = %Session.Cookie{key: \"key\", salt: \"salt\"}\n    iex> token = %{}\n    iex> token = Session.config(token, config)\n    iex> token.session_config == config\n    true","ref":"Aino.Session.html#config/2"},{"type":"function","title":"Aino.Session.decode/1","doc":"Decode session data from the token\n\nCan only be used with `Aino.Session.config/2` having run before.\n\nThe following keys will be added to the token `[:session]`","ref":"Aino.Session.html#decode/1"},{"type":"function","title":"Aino.Session.encode/1","doc":"Encode session data from the token\n\nCan only be used with `Aino.Middleware.cookies/1` and `Aino.Session.config/2` having run before.","ref":"Aino.Session.html#encode/1"},{"type":"module","title":"Aino.Session.Cookie","doc":"Session implementation using cookies as the storage","ref":"Aino.Session.Cookie.html"},{"type":"function","title":"Aino.Session.Cookie.decode/2","doc":"Parse session data from cookies\n\nVerifies the signature and if valid, parses session JSON data.\n\nCan only be used with `Aino.Middleware.cookies/1` and `Aino.Session.config/2` having run before.\n\nAdds the following keys to the token `[:session]`","ref":"Aino.Session.Cookie.html#decode/2"},{"type":"function","title":"Aino.Session.Cookie.encode/2","doc":"Response will be returned with two new `Set-Cookie` headers, a signature\nof the session data and the session data itself as JSON.","ref":"Aino.Session.Cookie.html#encode/2"},{"type":"module","title":"Aino.Session.EncryptedCookie","doc":"Session implementation using cookies as the storage","ref":"Aino.Session.EncryptedCookie.html"},{"type":"function","title":"Aino.Session.EncryptedCookie.decode/2","doc":"Parse encrypted session data from cookies.\n\nCan only be used with `Aino.Middleware.cookies/1` and `Aino.Session.config/2` having run before.\n\nRequires the :key set in the :config map to be exactly 256 bits (32 bytes).\n\nAdds the following keys to the token `[:session]`","ref":"Aino.Session.EncryptedCookie.html#decode/2"},{"type":"function","title":"Aino.Session.EncryptedCookie.encode/2","doc":"Response will be returned with one new `Set-Cookie` headers\nwith the session data as a JSON encoded, encrypted, base64\nencoded string","ref":"Aino.Session.EncryptedCookie.html#encode/2"},{"type":"module","title":"Aino.Session.Flash","doc":"A temporary storage for strings\n\nPrimarily to display notices on the next page load, e.g. \"Success!\" after a POST","ref":"Aino.Session.Flash.html"},{"type":"function","title":"Aino.Session.Flash.get/2","doc":"Fetch a key from the loaded flash message\n\nCan only be used with `Aino.Session.Flash.load/1`","ref":"Aino.Session.Flash.html#get/2"},{"type":"function","title":"Aino.Session.Flash.load/1","doc":"Load flash messages from session storage\n\nFlash messages in the session are deleted after being loaded.\n\n_Must_ be after `Aino.Session.decode/1`","ref":"Aino.Session.Flash.html#load/1"},{"type":"function","title":"Aino.Session.Flash.put/3","doc":"Set a temporary flash message\n\n_Must_ be after `Aino.Session.decode/1`","ref":"Aino.Session.Flash.html#put/3"},{"type":"protocol","title":"Aino.Session.Storage","doc":"Encode and decode session data in a pluggable backend","ref":"Aino.Session.Storage.html"},{"type":"function","title":"Aino.Session.Storage.decode/2","doc":"Parse session data on the token\n\nThe following keys should be added to the token `[:session]`","ref":"Aino.Session.Storage.html#decode/2"},{"type":"function","title":"Aino.Session.Storage.encode/2","doc":"Set session data from the token","ref":"Aino.Session.Storage.html#encode/2"},{"type":"type","title":"Aino.Session.Storage.t/0","doc":"All the types that implement this protocol.","ref":"Aino.Session.Storage.html#t:t/0"},{"type":"module","title":"Aino.Session.Token","doc":"Token functions related only to session\n\nSession data _must_ be decoded before using these functions","ref":"Aino.Session.Token.html"},{"type":"function","title":"Aino.Session.Token.clear/1","doc":"Clear a session, resetting to an empty map","ref":"Aino.Session.Token.html#clear/1"},{"type":"function","title":"Aino.Session.Token.delete/2","doc":"Delete a key from the session\n\nWhile keeping the rest of the session in tact","ref":"Aino.Session.Token.html#delete/2"},{"type":"function","title":"Aino.Session.Token.delete/3","doc":"","ref":"Aino.Session.Token.html#delete/3"},{"type":"function","title":"Aino.Session.Token.put/3","doc":"Puts a new key/value session data\n\nValues _must_ be serializable via JSON.\n\nSession data _must_ be decoded before using putting a new key","ref":"Aino.Session.Token.html#put/3"},{"type":"exception","title":"Aino.View.MissingTemplateException","doc":"","ref":"Aino.View.MissingTemplateException.html"},{"type":"function","title":"Aino.View.MissingTemplateException.message/1","doc":"","ref":"Aino.View.MissingTemplateException.html#message/1"},{"type":"extras","title":"Aino","doc":"# Aino\n\n[![Discord](https://img.shields.io/badge/chat-discord-7289da.svg)](https://discord.gg/wVDSWJ2EjE)\n\nAn experimental HTTP framework built on top of [elli][elli]. Aino is pronounced as \"eye no\".","ref":"readme.html"},{"type":"extras","title":"Why Aino? - Aino","doc":"Aino is an experiment to try out a new way of writing HTTP applications on Elixir. It uses [elli][elli] instead of Cowboy like Phoenix and Plug. Instead of writing an Endpoint like Phoenix, you write a Handler. The handler's job is to reduce across a series of middleware that are simple functions to generate a response.\n\nThe handler also works on a token instead of a conn. The token is a simple map that you can add whatever keys you wish to it. Aino has a few standard keys but you can easily ignore them if you want to write your own processing.","ref":"readme.html#why-aino"},{"type":"extras","title":"How to use Aino - Aino","doc":"In order to use Aino, you must add it to your supervision tree and provide a callback handler that Aino will call `handle/1` on.\n\n```elixir\ndefmodule Aino.Application do\n  use Application\n\n  def start(_type, _args) do\n    # get your config somehow\n\n    aino_config = %Aino.Config{\n      callback: Example.Web.Handler,\n      otp_app: :example,\n      host: config.host,\n      port: config.port,\n      environment: config.environment,\n      config: %{}\n    }\n\n    children = [\n      {Aino.Supervisor, aino_config}\n    ]\n\n    opts = [strategy: :one_for_one, name: Aino.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nIn the handler, you process the incoming request (in the `token`) through a series of \"middleware.\" The middleware all accept a single parameter, the `token`. A `token` is simply a map that you can store whatever you want on it.\n\nThe only thing that is initially pased in is the `:request`, and at the very end of the `handle/1` the token should include three keys, `:response_status`, `:response_headers`, and `:response_body`.\n\nAino ships with a common set of middleware that you can include at the top of processing, if you don't want them, simply don't include them! The list of middleware can be a list of lists as well.\n\nAnother built in middleware is a simple routing layer. Import the HTTP methods from `Aino.Middleware.Routes` that you're going to use in your routes. Then each HTTP method function takes the route and a middleware that should be run on the route.\n\n```elixir\ndefmodule MyApp.Handler do\n  import Aino.Middleware.Routes, only: [get: 2, get: 3, post: 2]\n\n  @behaviour Aino.Handler\n\n  def routes() do\n    [\n      get(\"/\", &Index.index/1, as: :root),\n      get(\"/about\", &Index.about/1, as: :about),\n      order_routes()\n    ]\n  end\n\n  defp order_routes() do\n    [\n      get(\"/orders\", &Orders.index/1, as: :orders),\n      get(\"/orders/:id\", &Orders.show/1, as: :order),\n      post(\"/orders\", &Orders.create/1)\n    ]\n  end\n\n  @impl true\n  def handle(token) do\n    middleware = [\n      Aino.Middleware.common(),\n      &Aino.Middleware.Routes.routes(&1, routes()),\n      &Aino.Middleware.Routes.match_route/1,\n      &Aino.Middleware.params/1,\n      &Aino.Middleware.Routes.handle_route/1,\n    ]\n\n    Aino.Token.reduce(token, middleware)\n  end\nend\n```\n\nThe route middleware take a token and generally should return the three keys required to render a response. You can also render EEx templates as shown below.\n\n```elixir\ndefmodule Index do\n  alias Aino.Token\n\n  def index(token) do\n    token\n    |> Token.response_status(200)\n    |> Token.response_header(\"Content-Type\", \"text/html\")\n    |> Token.response_body(Index.View.render(\"index.html\"))\n  end\nend\n\ndefmodule Index.View do\n  require Aino.View\n\n  Aino.View.compile [\n    \"lib/index/index.html.eex\"\n  ]\nend\n```","ref":"readme.html#how-to-use-aino"},{"type":"extras","title":"Concepts - Aino","doc":"### `Aino.Handler`\n\nA handler processes an incoming request from Aino.\n\nThe `handle/1` function is passed an `Aino.Token`.\n\nThe handler _must_ return a token that contains three keys to return a response:\n\n- `:response_status`\n- `:response_headers`\n- `:response_body`\n\nIf the token does not contain these three keys, a 500 error is returned.\n\nInside your handler, you may wish to use several `Aino.Middleware` including\n`Aino.Middleware.common/0`.\n\n### `Aino.Token`\n\nThe token is what flows through the entire web request. Tokens are simple maps\nthat contain no defined keys beyond `:request`. Several Aino middleware add\nkeys and they are documented in the functions.\n\n\n### `Aino.Middleware`\n\nMiddleware are simple functions that take the token and return the token. They process\nthe request and add or modify existing keys on the token.\n\nAn example middleware is `Aino.Middleware.headers/1`:\n\n```elixir\ndef headers(%{request: request} = token) do\n  headers =\n    Enum.map(request.headers, fn {header, value} ->\n      {String.downcase(header), value}\n    end)\n\n  Map.put(token, :headers, headers)\nend\n```\n\n[elli]: https://github.com/elli-lib/elli","ref":"readme.html#concepts"}],"content_type":"text/markdown"}